==> atmos-2.96.7.tar.gz: atmos-2.96.7/src/background_factory.cpp <==
/*
    $Id: background_factory.cpp 80 2010-02-21 16:20:54Z matwey $
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "background_factory.h"

#include <boost/format.hpp>

background_factory::background_factory(const select_model_function_type& select_model):
	m_select_model(select_model),
	m_curmodel(0) {
}
background_factory::background_factory(const background_factory& factory) {
	typedef filter_list_type::const_iterator filter_iterator;

	for( filter_iterator it = factory.m_filter_list.begin(); it != factory.m_filter_list.end(); ++it ) {
		m_filter_list.push_back( (*it)->clone() );
	}

	m_select_model = factory.m_select_model;
	m_curmodel = std::auto_ptr<model_type>(factory.m_curmodel->clone());
}
void background_factory::swap(background_factory& factory) throw() {
	factory.m_filter_list.swap( m_filter_list );
	factory.m_select_model.swap( m_select_model );
	std::swap( factory.m_curmodel, m_curmodel );
}
background_factory& background_factory::operator=(const background_factory& factory){
	background_factory tmp( factory );
	tmp.swap(*this );
	return *this;
}
background_factory::~background_factory() {
	typedef filter_list_type::iterator filter_iterator;
	
	for( filter_iterator it = m_filter_list.begin(); it != m_filter_list.end(); ++it ) {
		delete (*it);
	}
}
background_factory::vector_type background_factory::background( const boost::posix_time::ptime& time ) const {
	if( ! m_curmodel.get() )
		throw std::logic_error("There is not any background model");
	return m_curmodel->background( time );
}
bool background_factory::filter( const boost::posix_time::ptime& time, const flux_type& mean, const vars_type& eps ) const {
	typedef filter_list_type::const_iterator const_iterator;
	
	bool filter = true;
	for( const_iterator it = m_filter_list.begin(); it != m_filter_list.end(); ++it ) {
		filter &= (*it)->filter( time, mean, eps ); // can throw here
	}
	return filter;
}
background_factory::model_type* background_factory::create_new_model( const boost::posix_time::ptime& time ) const {
	return m_select_model( time );
}
background_factory::model_type* background_factory::release_current_model() {
	return m_curmodel.release();
}
void background_factory::handle_new_model( model_type* model ) {
	m_curmodel = std::auto_ptr<model_type>( model );
}

background_model::background_model( const boost::posix_time::ptime& time ): m_inittime(time) {
}
background_model::~background_model() {
}
background_abstract_filter::background_abstract_filter() {
}
background_abstract_filter::~background_abstract_filter() {
}

background_daytime_model::background_daytime_model( const boost::posix_time::ptime& time ):
	m_size(0),
	background_model(time) {
}
background_daytime_model::size_type background_daytime_model::size() const {
	return m_size;
}
background_daytime_model::vector_type background_daytime_model::background( const boost::posix_time::ptime& time ) const {
	throw std::logic_error("Daytime background observation");
}
void background_daytime_model::accumulate( const boost::posix_time::ptime& time, const vector_type& mean ) {
	m_size++;
}
background_model* background_daytime_model::clone() const {
	return new background_daytime_model(*this);
}

background_twilight_model::background_twilight_model( const boost::posix_time::ptime& time, const sun_altitude_function_type& sun_altitude ):
	m_size(0),
	m_background(0),
	m_sun_altitude(sun_altitude),
	background_model(time) {
}
background_twilight_model::value_type background_twilight_model::extrapolate( const boost::posix_time::ptime& timef ) const {
	value_type hi = m_sun_altitude( inittime() ) * 180.0 / M_PI; // degrees
	value_type hf = m_sun_altitude( timef ) * 180.0 / M_PI; // degrees
	if( hi < -12.0 ) hi = -12.0;
	if( hf < -12.0 ) hf = -12.0;
	value_type scale = 0.52189 - 0.26242*hi - 0.0235464*hi*hi; // model
	return std::exp( scale * ( hf - hi ) );
}
background_twilight_model::size_type background_twilight_model::size() const {
	return m_size;
}
background_twilight_model::vector_type background_twilight_model::background( const boost::posix_time::ptime& time ) const {
	return m_background * extrapolate( time );
}
void background_twilight_model::accumulate( const boost::posix_time::ptime& time, const vector_type& mean ) {
	// FIXME: 1. nonlinear correct
	
	vector_type corrected = mean / extrapolate( time );
	
	if( m_size == 0 ) {
		m_background = corrected;
		m_size++;
		return;
	}

	assert( corrected.size() == m_background.size() );

	m_background = m_background * ( value_type( m_size ) / value_type( m_size + 1 ) ) + corrected * ( value_type(1) / value_type( m_size + 1 ) );
	m_size++;
}
background_model* background_twilight_model::clone() const {
	return new background_twilight_model(*this);
}

background_night_model::background_night_model( const boost::posix_time::ptime& time ):
	m_size(0),
	m_background(0),
	background_model(time) {
}
background_night_model::size_type background_night_model::size() const {
	return m_size;
}
background_night_model::vector_type background_night_model::background( const boost::posix_time::ptime& time ) const {
	return m_background;
}
void background_night_model::accumulate( const boost::posix_time::ptime& time, const vector_type& mean ) {
	if( m_size == 0 ) {
		m_background = mean;
		m_size++;
		return;
	}

	assert( mean.size() == m_background.size() );

	m_background = m_background * ( value_type( m_size ) / value_type( m_size + 1 ) ) + mean * ( value_type(1) / value_type( m_size + 1 ) );
	m_size++;
}
background_model* background_night_model::clone() const {
	return new background_night_model(*this);
}

background_nonpoisson_filter::background_nonpoisson_filter( value_type threshold, const nonpoissonD_function_type& nonpoissonD, const numexp_function_type& numexp ): m_threshold(threshold), m_nonpoissonD(nonpoissonD), m_numexp(numexp) {
}
background_nonpoisson_filter::~background_nonpoisson_filter() {
}
bool background_nonpoisson_filter::filter( const boost::posix_time::ptime& time, const flux_type& mean, const vars_type& eps ) const {
	double poisson = ( eps(3,3)*mean(3) );
	double relepsp = 2.0 * ( 1.0 + 1.0/mean(3) ) / ( m_numexp() );
	double pmax = m_nonpoissonD() * ( 1.0 + m_threshold * std::sqrt(relepsp) );
	if( poisson > pmax ) {
		throw std::logic_error( (boost::format("Background nonpoisson %.4f is over the threshold %.4f") % poisson % pmax).str() );
		return false;
	}
	return true;
}
background_abstract_filter* background_nonpoisson_filter::clone() const {
	return new background_nonpoisson_filter( *this );
}

==> atmos-2.96.9.tar.gz: atmos-2.96.9/src/background_factory.cpp <==
/*
    $Id: background_factory.cpp 147 2010-04-17 09:25:25Z matwey $
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "background_factory.h"

#include <boost/format.hpp>

background_factory::background_factory(const select_model_function_type& select_model):
	m_curmodel(0),
	m_select_model(select_model) {
}
background_factory::background_factory(const background_factory& factory) {
	m_select_model = factory.m_select_model;
	m_curmodel = std::auto_ptr<model_type>(factory.m_curmodel->clone());
}
void background_factory::swap(background_factory& factory) throw() {
	factory.m_select_model.swap( m_select_model );
	std::swap( factory.m_curmodel, m_curmodel );
}
background_factory& background_factory::operator=(const background_factory& factory){
	background_factory tmp( factory );
	tmp.swap(*this );
	return *this;
}
background_factory::~background_factory() {
}
background_factory::vector_type background_factory::background( const boost::posix_time::ptime& time ) const {
	if( ! m_curmodel.get() )
		throw std::logic_error("There is not any background model");
	return m_curmodel->background( time );
}
background_factory::model_type* background_factory::create_new_model( const boost::posix_time::ptime& time ) const {
	return m_select_model( time );
}
background_factory::model_type* background_factory::release_current_model() {
	return m_curmodel.release();
}
void background_factory::handle_new_model( model_type* model ) {
	m_curmodel = std::auto_ptr<model_type>( model );
}

background_model::background_model( const boost::posix_time::ptime& time ): m_inittime(time) {
}
background_model::~background_model() {
}

background_daytime_model::background_daytime_model( const boost::posix_time::ptime& time ):
	background_model(time),
	m_size(0) {
}
background_daytime_model::size_type background_daytime_model::size() const {
	return m_size;
}
background_daytime_model::vector_type background_daytime_model::background( const boost::posix_time::ptime& time ) const {
	throw std::logic_error("Daytime background observation");
}
void background_daytime_model::accumulate( const boost::posix_time::ptime& time, const vector_type& mean ) {
	m_size++;
}
background_model* background_daytime_model::clone() const {
	return new background_daytime_model(*this);
}

background_twilight_model::background_twilight_model( const boost::posix_time::ptime& time, const sun_altitude_function_type& sun_altitude ):
	background_model(time),
	m_size(0),
	m_background(0),
	m_sun_altitude(sun_altitude) {
}
background_twilight_model::value_type background_twilight_model::extrapolate( const boost::posix_time::ptime& timef ) const {
	value_type hi = m_sun_altitude( inittime() ) * 180.0 / M_PI; // degrees
	value_type hf = m_sun_altitude( timef ) * 180.0 / M_PI; // degrees
	if( hi < -12.0 ) hi = -12.0;
	if( hf < -12.0 ) hf = -12.0;
	value_type scale = 0.52189 - 0.26242*hi - 0.0235464*hi*hi; // model
	return std::exp( scale * ( hf - hi ) );
}
background_twilight_model::size_type background_twilight_model::size() const {
	return m_size;
}
background_twilight_model::vector_type background_twilight_model::background( const boost::posix_time::ptime& time ) const {
	return m_background * extrapolate( time );
}
void background_twilight_model::accumulate( const boost::posix_time::ptime& time, const vector_type& mean ) {

	if( m_size == 0 ) {
		m_background = mean / extrapolate( time );
		m_size++;
		return;
	}

	m_background = m_background * ( value_type( m_size ) / value_type( m_size + 1 ) ) + mean / extrapolate( time ) * ( value_type(1) / value_type( m_size + 1 ) );
	m_size++;
}
background_model* background_twilight_model::clone() const {
	return new background_twilight_model(*this);
}

background_night_model::background_night_model( const boost::posix_time::ptime& time ):
	background_model(time),
	m_size(0),
	m_background(0) {
}
background_night_model::size_type background_night_model::size() const {
	return m_size;
}
background_night_model::vector_type background_night_model::background( const boost::posix_time::ptime& time ) const {
	return m_background;
}
void background_night_model::accumulate( const boost::posix_time::ptime& time, const vector_type& mean ) {
	if( m_size == 0 ) {
		m_background = mean;
		m_size++;
		return;
	}

	assert( mean.size() == m_background.size() );

	m_background = m_background * ( value_type( m_size ) / value_type( m_size + 1 ) ) + mean * ( value_type(1) / value_type( m_size + 1 ) );
	m_size++;
}
background_model* background_night_model::clone() const {
	return new background_night_model(*this);
}

background_nonpoisson_filter::background_nonpoisson_filter( const value_type& threshold, const value_type& nonpoissonD, const value_type& numexp ):
	m_threshold( threshold ),
	m_nonpoissonD( nonpoissonD ),
	m_numexp( numexp ) {
}
background_nonpoisson_filter::~background_nonpoisson_filter() {
}
bool background_nonpoisson_filter::operator() ( const boost::posix_time::ptime& time, const flux_type& flux, const vars_type& vars ) const {
	double poisson = ( vars(3,3)*flux(3) );
	double relepsp = 2.0 * ( 1.0 + 1.0/flux(3) ) / ( m_numexp );
	double pmax = m_nonpoissonD * ( 1.0 + m_threshold * std::sqrt(relepsp) );
	if( poisson > pmax ) {
		throw std::logic_error( (boost::format("Background nonpoisson %.4f is over the threshold %.4f") % poisson % pmax).str() );
		return true;
	}
	return false;
}
